@@ -8,7 +8,6 @@
 import os
 import sys
 import traceback
-import logging
 from importlib import import_module
 from collections import OrderedDict
 
@@ -17,10 +16,11 @@ import six
 
 from law.config import Config
 from law.task.base import Task, ExternalTask
-from law.util import multi_match, colored, abort
+from law.util import multi_match, colored, abort, makedirs
+from law.logger import get_logger
 
 
-logger = logging.getLogger(__name__)
+logger = get_logger(__name__)
 
 _cfg = Config.instance()
 
@@ -41,13 +41,19 @@ def setup_parser(sub_parsers):
         "file and exit")
     parser.add_argument("--location", "-l", action="store_true", help="print the location of the "
         "index file and exit")
-    parser.add_argument("--verbose", "-v", action="store_true", help="verbose output")
+    parser.add_argument("--quiet", "-q", action="store_true", help="quiet mode without output")
+    parser.add_argument("--verbose", "-v", action="store_true", help="verbose output, disables the "
+        "quiet mode when set")
 
 
 def execute(args):
     """
     Executes the *index* subprogram with parsed commandline *args*.
     """
+    # update args
+    if args.verbose:
+        args.quiet = False
+
     cfg = Config.instance()
     index_file = cfg.get_expanded("core", "index_file")
 
@@ -77,7 +83,8 @@ def execute(args):
     if args.modules:
         lookup += args.modules
 
-    print("indexing tasks in {} module(s)".format(len(lookup)))
+    if not args.quiet:
+        print("indexing tasks in {} module(s)".format(len(lookup)))
 
     # loop through modules, import everything to load tasks
     for modid in lookup:
@@ -108,12 +115,6 @@ def execute(args):
         cls = lookup.pop(0)
         lookup.extend(cls.__subclasses__())
 
-        # skip already seen task families
-        task_family = cls.get_task_family()
-        if task_family in seen_families:
-            continue
-        seen_families.append(task_family)
-
         # skip tasks in __main__ module in interactive sessions
         if cls.__module__ == "__main__":
             continue
@@ -136,6 +137,7 @@ def execute(args):
         # show an error when there is a "-" in the task family as the luigi command line parser will
         # automatically map it to "_", i.e., it will fail to lookup the actual task class
         # skip the task
+        task_family = cls.get_task_family()
         if "-" in task_family:
             logger.critical("skipping task '{}' as its family '{}' contains a '-' which cannot be "
                 "interpreted by luigi's command line parser, please use '_' or alike".format(
@@ -152,6 +154,18 @@ def execute(args):
                 "parameters in the law shell autocompletion".format(cls, task_family))
             continue
 
+        # skip already seen task families and warn when the class is not added yet in the classes to
+        # index as this is usually a sign of multiple definitions of the same task, e.g. through
+        # imports of the same physical file via different module ids
+        if task_family in seen_families:
+            if cls not in task_classes:
+                logger.error("skipping task '{}' as a task with the same family '{}' but a "
+                    "different address was already seen; this is likely due to multiple imports of "
+                    "the same physical file through different module ids and since it is no longer "
+                    "unique, luigi's task lookup will probably fail".format(cls, task_family))
+            continue
+        seen_families.append(task_family)
+
         task_classes.append(cls)
 
     def get_task_params(cls):
@@ -171,8 +185,7 @@ def execute(args):
     stats = OrderedDict()
 
     # write the index file
-    if not os.path.exists(os.path.dirname(index_file)):
-        os.makedirs(os.path.dirname(index_file))
+    makedirs(os.path.dirname(index_file))
 
     with open(index_file, "w") as f:
         for cls in task_classes:
@@ -194,7 +207,8 @@ def execute(args):
                 print("    - {}".format(colored(task_family, "green")))
         print("")
 
-    print("written {} task(s) to index file '{}'".format(len(task_classes), index_file))
+    if not args.quiet:
+        print("written {} task(s) to index file '{}'".format(len(task_classes), index_file))
 
 
 def get_global_parameters(config_names=("core", "scheduler", "worker", "retcode")):
