@@ -85,6 +85,7 @@ class Config(ConfigParser):
             "colored_repr": False,
             "colored_str": True,
             "expand_path_repr": False,
+            "filesize_repr": False,
             "default_local_fs": "local_fs",
             "tmp_dir": os.getenv("LAW_TARGET_TMP_DIR") or tempfile.gettempdir(),
             "tmp_dir_perm": 0o0770,
@@ -213,6 +214,11 @@ class Config(ConfigParser):
             "lsf_job_file_dir_cleanup": False,
             "lsf_chunk_size_cancel": 20,
             "lsf_chunk_size_query": 20,
+            "slurm_job_file_dir": None,
+            "slurm_job_file_dir_mkdtemp": None,
+            "slurm_job_file_dir_cleanup": False,
+            "slurm_chunk_size_cancel": 20,
+            "slurm_chunk_size_query": 20,
         },
         "notifications": {
             "mail_recipient": None,
@@ -233,6 +239,11 @@ class Config(ConfigParser):
             "login": False,
         },
         "bash_sandbox_env": {},
+        "venv_sandbox": {
+            "stagein_dir_name": "stagein",
+            "stageout_dir_name": "stageout",
+        },
+        "venv_sandbox_env": {},
         "docker_sandbox": {
             "stagein_dir_name": "stagein",
             "stageout_dir_name": "stageout",
