@@ -5,9 +5,14 @@
 HTCondor contrib functionality.
 """
 
-__all__ = ["HTCondorJobManager", "HTCondorJobFileFactory", "HTCondorWorkflow"]
+__all__ = [
+    "get_htcondor_version",
+    "HTCondorJobManager", "HTCondorJobFileFactory",
+    "HTCondorWorkflow",
+]
 
 
 # provisioning imports
+from law.contrib.htcondor.util import get_htcondor_version
 from law.contrib.htcondor.job import HTCondorJobManager, HTCondorJobFileFactory
 from law.contrib.htcondor.workflow import HTCondorWorkflow
