@@ -5,9 +5,14 @@
 LSF contrib functionality.
 """
 
-__all__ = ["LSFJobManager", "LSFJobFileFactory", "LSFWorkflow"]
+__all__ = [
+    "get_lsf_version",
+    "LSFJobManager", "LSFJobFileFactory",
+    "LSFWorkflow",
+]
 
 
 # provisioning imports
+from law.contrib.lsf.util import get_lsf_version
 from law.contrib.lsf.job import LSFJobManager, LSFJobFileFactory
 from law.contrib.lsf.workflow import LSFWorkflow
