@@ -101,7 +101,7 @@ def hadd_task(task, inputs, output, cwd=None, local=False, force=True, hadd_args
                     raise Exception("hadd failed")
 
         task.publish_message("merged file size: {}".format(human_bytes(
-            output.stat.st_size, fmt=True)))
+            output.stat().st_size, fmt=True)))
 
     else:
         # when not local, we need to fetch files first into the cwd
@@ -129,4 +129,4 @@ def hadd_task(task, inputs, output, cwd=None, local=False, force=True, hadd_args
                         raise Exception("hadd failed")
 
                     task.publish_message("merged file size: {}".format(human_bytes(
-                        tmp_out.stat.st_size, fmt=True)))
+                        tmp_out.stat().st_size, fmt=True)))
