@@ -19,7 +19,6 @@ invocation (``@log()``), for law to distuinguish between the two cases **always*
 arguments when configuring decorators. Default arguments are applied in either case.
 """
 
-
 __all__ = [
     "factory", "log", "safe_output", "delay", "notify", "timeit", "localize", "require_sandbox",
 ]
@@ -34,9 +33,9 @@ import random
 import socket
 import collections
 import uuid
-import logging
 
 import luigi
+import six
 
 from law.task.proxy import ProxyTask
 from law.sandbox.base import SandboxTask
@@ -44,11 +43,13 @@ from law.parameter import get_param, NotifyParameter
 from law.target.file import localize_file_targets
 from law.target.local import LocalFileTarget
 from law.util import (
-    no_value, make_list, multi_match, human_duration, open_compat, join_generators, TeeStream,
+    no_value, uncolored, make_list, multi_match, human_duration, open_compat, join_generators,
+    TeeStream,
 )
+from law.logger import get_logger
 
 
-logger = logging.getLogger(__name__)
+logger = get_logger(__name__)
 
 
 class _CompleteTask(luigi.Task):
@@ -409,8 +410,25 @@ def notify(fn, opts, task, *args, **kwargs):
         for transport in transports:
             fn = transport["func"]
             raw = transport.get("raw", False)
+            colored = transport.get("colored", False)
+
+            # remove color commands if necessary
+            if not colored:
+                _title = uncolored(title)
+                if raw:
+                    _content = {
+                        k: (uncolored(v) if isinstance(v, six.string_types) else v)
+                        for k, v in parts.items()
+                    }
+                else:
+                    _content = uncolored(message)
+            else:
+                _title = title
+                _content = parts.copy() if raw else message
+
+            # invoke the function
             try:
-                fn(success, title, parts.copy() if raw else message, **opts)
+                fn(success, _title, _content, **opts)
             except Exception as e:
                 t = traceback.format_exc()
                 logger.warning("notification via transport '{}' failed: {}\n{}".format(fn, e, t))
@@ -424,12 +442,11 @@ def notify(fn, opts, task, *args, **kwargs):
     return before_call, call, after_call, on_error
 
 
-@factory(publish_message=False, accept_generator=True)
+@factory(accept_generator=True)
 def timeit(fn, opts, task, *args, **kwargs):
-    """ timeit(publish_message=False)
-    Wraps a bound method of a task and logs its execution time in a human readable format. Logs in
-    info mode. When *publish_message* is *True*, the duration is also published as a task message to
-    the scheduler. Accepts generator functions.
+    """ timeit()
+    Wraps a bound method of a task and logs its execution time in a human readable format using the
+    task's logger instance in info mode. Accepts generator functions.
     """
     def before_call():
         t0 = time.time()
@@ -440,14 +457,7 @@ def timeit(fn, opts, task, *args, **kwargs):
 
     def log_duration(t0):
         duration = human_duration(seconds=round(time.time() - t0, 1))
-
-        # log
-        timeit_logger = logger.getChild("timeit")
-        timeit_logger.info("runtime of {}: {}".format(task.task_id, duration))
-
-        # optionally publish a task message to the scheduler
-        if opts["publish_message"] and callable(getattr(task, "publish_message", None)):
-            task.publish_message("runtime: {}".format(duration))
+        task.logger.info("runtime: {}".format(duration))
 
     def after_call(t0):
         log_duration(t0)
