@@ -4,7 +4,6 @@
 Formatter classes for file targets.
 """
 
-
 __all__ = ["AUTO_FORMATTER", "Formatter", "get_formatter", "find_formatters", "find_formatter"]
 
 
@@ -12,15 +11,15 @@ import os
 import json
 import zipfile
 import tarfile
-import logging
 from collections import OrderedDict
 
 import six
 
 from law.util import make_list
+from law.logger import get_logger
 
 
-logger = logging.getLogger(__name__)
+logger = get_logger(__name__)
 
 
 AUTO_FORMATTER = "auto"
@@ -85,8 +84,7 @@ def find_formatter(path, mode, name=AUTO_FORMATTER):
         return get_formatter(name, silent=False)
 
 
-@six.add_metaclass(FormatterRegister)
-class Formatter(object):
+class Formatter(six.with_metaclass(FormatterRegister, object)):
 
     name = "_base"
 
@@ -181,7 +179,7 @@ class YAMLFormatter(Formatter):
         import yaml
 
         with open(get_path(path), "r") as f:
-            return yaml.load(f, *args, **kwargs)
+            return yaml.safe_load(f, *args, **kwargs)
 
     @classmethod
     def dump(cls, path, obj, *args, **kwargs):
