@@ -26,7 +26,7 @@ from law.parser import root_task, global_cmdline_values
 from law.logger import setup_logger
 from law.util import (
     no_value, abort, law_run, common_task_params, colored, uncolored, make_list, multi_match,
-    flatten, BaseStream, human_duration, patch_object, round_discrete,
+    flatten, BaseStream, human_duration, patch_object, round_discrete, empty_context,
 )
 from law.logger import get_logger
 
@@ -81,6 +81,7 @@ class BaseTask(six.with_metaclass(BaseRegister, luigi.Task)):
     exclude_params_req = set()
     exclude_params_req_set = set()
     exclude_params_req_get = set()
+    prefer_params_cli = set()
 
     @staticmethod
     def resource_name(name, host=None):
@@ -187,14 +188,15 @@ class BaseTask(six.with_metaclass(BaseRegister, luigi.Task)):
         params.update(kwargs)
 
         # remove params that are preferably set via cli class arguments
-        if _prefer_cli:
+        prefer_cli = set(cls.prefer_params_cli or ()) if _prefer_cli is None else set(_prefer_cli)
+        if prefer_cli:
             cls_args = []
             prefix = cls.get_task_family() + "_"
             if luigi.cmdline_parser.CmdlineParser.get_instance():
                 for key in global_cmdline_values().keys():
                     if key.startswith(prefix):
                         cls_args.append(key[len(prefix):])
-            for name in make_list(_prefer_cli):
+            for name in make_list(prefer_cli):
                 if name in params and name in cls_args:
                     del params[name]
 
@@ -469,6 +471,19 @@ class Task(six.with_metaclass(Register, BaseTask)):
         else:
             return make_callback(n_total, *reach)
 
+    def iter_progress(self, iterable, n_total, reach=(0, 100), precision=1, msg=None):
+        # create a progress callback with all arguments
+        progress_callback = self.create_progress_callback(n_total, reach=(0, 100), precision=1)
+
+        # when msg is set, place the iteration in an outer context
+        context = (lambda: self.publish_step(msg)) if msg else empty_context
+
+        # iterate and invoke the callback
+        with context():
+            for i, val in enumerate(iterable):
+                yield val
+                progress_callback(i)
+
     def cli_args(self, exclude=None, replace=None):
         exclude = set() if exclude is None else set(make_list(exclude))
 
