@@ -7,15 +7,15 @@ Helpful utility functions.
 __all__ = [
     "default_lock", "io_lock", "console_lock", "no_value", "rel_path", "law_src_path",
     "law_home_path", "law_run", "print_err", "abort", "is_number", "try_int", "round_discrete",
-    "str_to_int", "flag_to_bool", "common_task_params", "colored", "uncolored", "query_choice",
-    "is_pattern", "brace_expand", "range_expand", "range_join", "multi_match", "is_iterable",
-    "is_lazy_iterable", "make_list", "make_tuple", "make_unique", "is_nested", "flatten",
-    "merge_dicts", "which", "map_verbose", "map_struct", "mask_struct", "tmp_file",
+    "str_to_int", "flag_to_bool", "empty_context", "common_task_params", "colored", "uncolored",
+    "query_choice", "is_pattern", "brace_expand", "range_expand", "range_join", "multi_match",
+    "is_iterable", "is_lazy_iterable", "make_list", "make_tuple", "make_unique", "is_nested",
+    "flatten", "merge_dicts", "which", "map_verbose", "map_struct", "mask_struct", "tmp_file",
     "interruptable_popen", "readable_popen", "create_hash", "create_random_string", "copy_no_perm",
     "makedirs", "user_owns_file", "iter_chunks", "human_bytes", "parse_bytes", "human_duration",
-    "human_time_diff", "parse_duration", "is_file_exists_error", "send_mail", "DotDict",
-    "ShorthandDict", "open_compat", "patch_object", "join_generators", "quote_cmd",
-    "escape_markdown", "classproperty", "BaseStream", "TeeStream", "FilteredStream",
+    "parse_duration", "is_file_exists_error", "send_mail", "DotDict", "ShorthandDict",
+    "open_compat", "patch_object", "join_generators", "quote_cmd", "escape_markdown",
+    "classproperty", "BaseStream", "TeeStream", "FilteredStream",
 ]
 
 
@@ -243,6 +243,15 @@ def flag_to_bool(s, silent=False):
         raise ValueError("cannot convert to bool: {}".format(s))
 
 
+@contextlib.contextmanager
+def empty_context():
+    """
+    Yields an empty context that can be used in case of dynamically choosing context managers while
+    maintaining code structure.
+    """
+    yield
+
+
 def common_task_params(task_instance, task_cls):
     """
     Returns the parameters that are common between a *task_instance* and a *task_cls* in a
@@ -1379,13 +1388,16 @@ def parse_bytes(s, input_unit="bytes", unit="bytes"):
 
 
 time_units = collections.OrderedDict([
-    ("day", 86400),
-    ("hour", 3600),
+    ("week", 7 * 24 * 60 * 60),
+    ("day", 24 * 60 * 60),
+    ("hour", 60 * 60),
     ("minute", 60),
     ("second", 1),
 ])
 
 time_unit_aliases = {
+    "w": "week",
+    "weeks": "week",
     "d": "day",
     "days": "day",
     "h": "hour",
@@ -1404,7 +1416,7 @@ time_unit_aliases = {
 def human_duration(colon_format=False, plural=True, **kwargs):
     """ human_duration
     Returns a human readable duration. The largest unit is days. When *colon_format* is *True*, the
-    return value has the format ``"[d:][hh:]mm:ss[.ms]"``. *colon_format* can also be a string value
+    return value has the format ``"[d-][hh:]mm:ss[.ms]"``. *colon_format* can also be a string value
     referring to a limiting  unit. In that case, the returned time string has no field above that
     unit, e.g. passing ``"m"`` results in a string ``"mm:ss[.ms]"`` where the minute field is
     potentially larger than 60. Passing ``"s"`` is a special case. Since the colon format always has
@@ -1428,7 +1440,7 @@ def human_duration(colon_format=False, plural=True, **kwargs):
     # -> "-20:33"
 
     human_duration(seconds=90001, colon_format=True)
-    # -> "1:01:00:01"
+    # -> "1-01:00:01"
 
     human_duration(seconds=90001, colon_format="h")
     # -> "25:00:01"
@@ -1448,6 +1460,8 @@ def human_duration(colon_format=False, plural=True, **kwargs):
     human_duration(minutes=-15)
     # -> "minus 15 minutes"
     """
+    _time_units = ["day", "hour", "minute", "second"]
+
     seconds = float(datetime.timedelta(**kwargs).total_seconds())
     sign = 1 if seconds >= 0 else -1
     seconds = abs(seconds)
@@ -1456,13 +1470,15 @@ def human_duration(colon_format=False, plural=True, **kwargs):
     colon_unit_limit = None
     if isinstance(colon_format, six.string_types):
         colon_unit_limit = time_unit_aliases.get(colon_format, colon_format)
-        if colon_unit_limit not in time_units:
+        if colon_unit_limit not in _time_units:
             raise ValueError("unknown colon_format unit '{}', valid values are {}".format(
-                colon_unit_limit, ",".join(time_units)))
-        colon_unit_index = list(time_units.keys()).index(colon_unit_limit)
+                colon_unit_limit, ",".join(_time_units)))
+        colon_unit_index = _time_units.index(colon_unit_limit)
 
-    parts = []
-    for i, (unit, mul) in enumerate(six.iteritems(time_units)):
+    # start building the human readable string
+    # loop through units, remove the fully dividable part and let the next unit handle the rest
+    human_str = ""
+    for i, unit in enumerate(_time_units):
         # skip this iteration when a colon unit limit is set
         if colon_unit_limit and i < colon_unit_index:
             continue
@@ -1473,55 +1489,42 @@ def human_duration(colon_format=False, plural=True, **kwargs):
             value = try_int(round(seconds, 2))
         else:
             # get the integer divider and adjust the remaining number of seconds
+            mul = time_units[unit]
             value = int(seconds // mul)
             seconds -= value * mul
 
-        # skip zeros under certain conditions
-        if not value:
-            leading = not parts
+        # keep zeros under certain conditions
+        if value == 0:
             if colon_format:
-                # skip zeros when leading but not referring to minutes or seconds
-                if leading and unit not in ("minute", "second"):
-                    continue
+                keep_zero = human_str or unit == "second" or colon_unit_limit
             else:
-                # skip zeros always, except when leading and the unit is seconds
-                if not leading or unit != "second":
-                    continue
+                keep_zero = not human_str and unit == "second"
+            if not keep_zero:
+                continue
 
         # build the human readable representation
         if colon_format:
-            if unit == "second" and value < 10:
-                # special case for seconds to format floating points properly
-                fmt = "0{}"
+            if unit == "second":
+                # special case 1: force float formatting with optional leading 0
+                fmt = "0{}" if value < 10 else "{}"
+                # special case 2: when "minutes" are no there yet, prepend "00:"
+                if not human_str:
+                    fmt = "00:" + fmt
             elif unit in ["hour", "minute"]:
-                fmt = "{:02d}"
-            else:
-                fmt = "{}"
-            parts.append(fmt.format(value))
+                fmt = "{:02d}:"
+            else:  # day
+                fmt = "{}-"
+            human_str += fmt.format(value)
         else:
-            plural_postfix = "" if (not plural or value == 1) else "s"
-            parts.append("{} {}{}".format(value, unit, plural_postfix))
+            if human_str:
+                human_str += ", "
+            human_str += "{} {}{}".format(value, unit, "" if (value == 1 or not plural) else "s")
 
-    # special case: the minute field is mandatory for colon_format in any case
-    if colon_format and len(parts) == 1:
-        parts.insert(0, "00")
-
-    # denote negative values
-    sign_prefix = ""
+    # sign
     if sign == -1:
-        sign_prefix = "-" if colon_format else "minus "
+        human_str = ("-" if colon_format else "minus ") + human_str
 
-    return sign_prefix + (":" if colon_format else ", ").join(parts)
-
-
-def human_time_diff(*args, **kwargs):
-    """
-    Deprecated. Use :py:func:`human_duration` instead.
-    """
-    # deprecation warning until v0.1
-    logger.warning("law.util.human_time_diff is deprecated, use law.util.human_duration instead")
-
-    return human_duration(*args, **kwargs)
+    return human_str
 
 
 def parse_duration(s, input_unit="s", unit="s"):
@@ -1535,21 +1538,20 @@ def parse_duration(s, input_unit="s", unit="s"):
 
         # plain number
         parse_duration(100)
-        # -> 100.
+        # -> 100.0
 
         parse_duration(100, unit="min")
         # -> 1.667
 
         parse_duration(100, input_unit="min")
-        # -> 6000.
+        # -> 6000.0
 
         parse_duration(-100, input_unit="min")
-        # -> -6000.
+        # -> -6000.0
 
-        # string separated with ":", interpreted from the back as seconds, minutes, etc.
-        # input_unit is disregarded, unit works as above
+        # strings in the format [d-][h:][m:]s[.ms] are interpreted with input_unit disregarded
         parse_duration("2:1")
-        # -> 121.
+        # -> 121.0
 
         parse_duration("04:02:01.1")
         # -> 14521.1
@@ -1557,7 +1559,10 @@ def parse_duration(s, input_unit="s", unit="s"):
         parse_duration("04:02:01.1", unit="min")
         # -> 242.0183
 
-        # human-readable string, optionally multiple of them
+        parse_duration("0-4:2:1.1")
+        # -> 14521.1
+
+        # human-readable string, optionally multiple of them separated by comma
         # missing units are interpreted as input_unit, unit works as above
         parse_duration("10 mins")
         # -> 600.0
@@ -1593,7 +1598,7 @@ def parse_duration(s, input_unit="s", unit="s"):
             unit, ",".join(time_units)))
 
     sign = 1
-    duration_seconds = 0.
+    duration_seconds = 0.0
 
     # number or string?
     if isinstance(s, six.integer_types + (float,)):
@@ -1601,30 +1606,25 @@ def parse_duration(s, input_unit="s", unit="s"):
     else:
         s = s.strip()
 
-        # check the format
-        if "," not in s and ":" in s:
-            # colon format, "[d:][h:][m:]s"
-            unit_order = ["second", "minute", "hour", "day"]
+        # identify the format "[d-][h:][m:]s[.ms]" first
+        m = re.match(r"^([+-])?((((((\d+)-)?(\d+)):)?(\d+)):)?(\d+)(\.(\d*))?$", s)
+        if m:
+            sgn, d, h, m, s, ms = [m.group(i) for i in [1, 7, 8, 9, 10, 11]]
 
             # interpret leading "-" or "+" as the sign of the duration
-            if s[0] in "+-":
-                sign = 1 if s[0] == "+" else -1
-                s = s[1:]
+            if sgn == "-":
+                sign = -1
 
-            # split and check the number of parts
-            parts = s.split(":")
-            if len(parts) > len(unit_order):
-                raise ValueError("cannot parse duration string '{}', too many ':'".format(s))
-
-            # convert each part, starting from the back to match unit_order
-            for i, part in enumerate(parts[::-1]):
-                u = unit_order[i]
-                try:
-                    d = float(part.strip())
-                except ValueError as e:
-                    raise ValueError("cannot parse duration string '{}', {}".format(s, e))
-
-                duration_seconds += d * time_units[u]
+            # add to seconds
+            if d:
+                duration_seconds += float(d) * time_units["day"]
+            if h:
+                duration_seconds += float(h) * time_units["hour"]
+            if m:
+                duration_seconds += float(m) * time_units["minute"]
+            duration_seconds += float(s)
+            if ms:
+                duration_seconds += float(ms)
 
         else:
             # human readable format
@@ -1785,6 +1785,72 @@ class ShorthandDict(collections.OrderedDict):
             super(ShorthandDict, self).__setattr__(attr, value)
 
 
+class InsertableDict(collections.OrderedDict):
+    """
+    Subclass of *OrderedDict* that supports inserting elements before or after certain keys.
+    Example:
+
+    .. code-block:: python
+
+        d = InsertableDict(foo=123, bar=456)
+
+        d.insert_before("bar", "test", 999)
+        print(d)  # -> InsertableDict([('foo', 123), ('test', 999), ('bar', 456)])
+
+        d.insert_after("test", "foo", "new_value")
+        print(d)  # -> InsertableDict([('test', 999), ('foo', 'new_value'), ('bar', 456)])
+    """
+
+    def _insert(self, search_key, key, value, offset):
+        # when key is a list or dict and value is None, assume key refers to key-value pairs
+        if isinstance(key, (list, dict)) and value is None:
+            new_items = key.items() if isinstance(key, dict) else key
+            new_keys = [k for k, v in new_items]
+        else:
+            new_items = [(key, value)]
+            new_keys = [key]
+
+        # if the search key is not present, insert the new pairs and finish
+        if search_key not in self:
+            self.update(new_items)
+            return
+
+        # create a copy if the index
+        items = list(self.items())
+
+        # find the position where to insert
+        pos = items.index((search_key, self[search_key])) + offset
+
+        # construct the new items without duplicates
+        items = [
+            (k, v) for k, v in items[:pos]
+            if k not in new_keys
+        ] + new_items + [
+            (k, v) for k, v in items[pos:]
+            if k not in new_keys
+        ]
+
+        # rebuild the index
+        self.clear()
+        self.update(items)
+
+    def insert_before(self, before_key, key, value=None):
+        """
+        Inserts a *key* - *value* pair before the key *before_key*. When this key does not exist,
+        the new pair is added to the end. When *key* is list or dictionary and value is *None*,
+        multiple new values are inserted.
+        """
+        self._insert(before_key, key, value, 0)
+
+    def insert_after(self, after_key, key, value=None):
+        """
+        Inserts a *key* - *value* pair after the key *after_key*. When this key does not exist, the
+        new pair is added to the end. When *key* is list or dictionary and value is *None*,
+        multiple new values are inserted.
+        """
+        self._insert(after_key, key, value, 1)
+
+
 def open_compat(*args, **kwargs):
     """
     Polyfill for python's ``open`` factory, returning the plain ``open`` in python 3, and
